/*
 * Copyright (c) 2004-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.oc;

public class PathUtils {
	/**
	 * 
	 */
	public PathUtils() {
		super();
	}
	
	/**
	 * Attach a trailing slash on the directory if it is missing
	 * @param dir
	 * @return
	 */
	public static String patchDir(String dir) 
	{
		if(dir == null)
			return null;
		// VFS uses slashes
	    if(dir.lastIndexOf("/") == dir.length()-1)
	    	return dir;
	    else
	    	return dir + "/";
	}
	
	/**
	 * Remove trailing ending if present in path
	 * @param path
	 * @param ending
	 * @return modified string
	 */
	public static String removeTrailing(String path, String ending) {
		if(path == null || ending == null)
			return path;
		if(path.endsWith(ending))
			return path.substring(0, path.length()-ending.length());
		else
			return path;
	}
	
	/**
	 * Works on paths (using / as path sep), eg:
	 * "/foo/bar.html" -> "/foo/"
	 * "/foo/bar/" -> "/foo/bar/"
	 * "/foo/bar" -> "/foo/"
	 * "foo.html" -> null
	 * "/foo.html" -> "/"
	 * "/" ->  "/"
	 * @param path
	 * @return folder part of the path (ending with a slash)
	 */
	public static String folderFromPath(String path) {
		if(path == null)
			return null;
		int ndx = path.lastIndexOf("/");
		if(ndx == -1)
			return null; // this marks the empty directory path
		else if(ndx == path.length()-1)
			return path; // all path no filename
		else
			return path.substring(0, ndx+1); // incl. the last slash 
	}

	/**
	 * Strips the folder from path, eg:
	 * "/foo/bar.html" -> "bar.html"
	 * "/foo/bar/" -> null
	 * "/foo/bar" -> "bar"
	 * "foo" -> "foo"
	 * "/" ->  null
	 * @param path
	 * @return filename
	 */
	public static String filenameFromPath(String path) {
		if(path == null)
			return null;
		int ndx = path.lastIndexOf("/");
		if(ndx == -1)
			return path; // no directory part
		else if(ndx == path.length()-1)
			return null; // all path no filename
		else
			return path.substring(ndx+1); // return without slashes 
	}
	
	/**
	 * Returns the last part of a path. Eg.:
	 * "/foo/bar.html" -> "bar.html"
	 * "/foo/bar/" -> "bar"
	 * "/foo/bar" -> "bar"
	 * "/foo" -> "foo"
	 * "/" ->  ""
	 * @param path
	 * @return
	 */
	public static String lastPathPart(String path) {
		if(path == null)
			return null;
	    int ndx = path.lastIndexOf("/");
	    if(ndx == path.length()-1)
	      path = path.substring(0, ndx); // strip trailing slash and try again
	    ndx = path.lastIndexOf("/");
	    if(ndx == -1)
	    	return path;
	    else
	    	return path.substring(ndx+1);
	}
	
	/**
	 * Extract key from a path startLvel = 0, endLevel = 1 , incl = true:
	 * "/" = ""
	 * "/index.html" = "index"
	 * "/folder" = "folder"
	 * "/folder/" = "folder"
	 * "/folder/index.html" = "folder/index"
	 * "/folder/folder2" = "folder/folder2"
	 * "/folder/folder2/" = "folder/folder2"
	 * "/folder/folder2/index.html" = "folder/folder2"
	 * 
	 * @param path
	 * @param startLevel
	 * @param endLevel
	 * @param inclFilename
	 * @return
	 */
	public static String keyFromPath(String path, int startLevel, int endLevel, boolean inclFilename) {
		return keyFromPath(path, startLevel, endLevel, inclFilename, null);
	}
	public static String keyFromPath(String path, int startLevel, int endLevel, boolean inclFilename, String replace) {
		if(path==null)
			return null;
		if(path.equals("/"))
			return "";
		boolean isPath = path.endsWith("/");
		String[] parts = path.split("/");
		StringBuffer result = new StringBuffer();
		int i;
		for(i = 0; i < parts.length-1; i++) {
			if(i >= startLevel && (endLevel < 0 || i <= endLevel)) {
				String tmp = parts[i+1];
				if(tmp.length()>0) {
					if(i < endLevel && i < parts.length-2)
						result.append("/"+tmp);
					else if(isPath)
						result.append("/"+tmp);
					else if(inclFilename)
						result.append("/"+(tmp.lastIndexOf(".") >= 0 ? tmp.substring(0,tmp.lastIndexOf(".")) : tmp));
				}
			}
		}
		if(result.length() > 0)
			if(replace==null)
				return result.toString().substring(1);
			else
				return result.toString().substring(1).replaceAll("/", replace);
		else
			return "";
	}
	
	/**
	 * Typically used with "." as marker: "/foo/bar.gif" -> "/foo/bar"
	 * @param path
	 * @param marker
	 * @return 
	 */
	public static String stripExtension(String path, String marker) {
		if(path == null || marker == null)
			return path;
		int ndx = path.lastIndexOf(marker);
		if(ndx == -1)
			return path; // no extension found 
		else if(path.lastIndexOf("/") > ndx)
			return path; // marker is before the last slash, e.g. /foo.bar/folder/file
		else
			return path.substring(0, ndx);
	}
	
	/**
	 * Typically used with "." as marker: "/foo/bar.gif" -> "gif"
	 * @param path
	 * @param marker
	 * @return 
	 */
	public static String getExtension(String path, String marker) {
		if(path == null || marker == null)
			return path;
		int ndx = path.lastIndexOf(marker);
		if(ndx == -1)
			return null; // no extension found
		else if(ndx == path.length()-1)
			return ""; // it ends with the marker so we say it is an empty exentions
		else if(path.lastIndexOf("/") > ndx)
			return null; // marker is before the last slash, e.g. /foo.bar/folder/file
		else
			return path.substring(ndx+1); // return extension without the marker
	}
	
	/**
	 * returns true if uri is relative, ie it does not incl. doc root nor protocol
	 * @param uri
	 * @param ignoreAnchors - if true anchors are not considered to be relative.
	 * @return
	 */
	public static boolean isRelativeUri(String uri, boolean ignoreAnchors) {
		// these are the possibilities "http://ext.com/xxx", "www.foo.bar/xxx", "/x/xxx", "xxx", "../xxx", "#ddd"
		// number two is wrongly considered local, but it is kind of malformed anyway.
		if( uri.startsWith("/") ||
				(ignoreAnchors && uri.startsWith("#")) ||
				uriHasProtocol(uri) )
			return false;
		else
			return true;
	}
	
	public static boolean uriHasProtocol(String uri) {
		if(uri.indexOf("://") == -1)
			return false;
		else
			return true; // could check that it is known? http, https, ftp....
	}
	
	/**
	 * If uri is a folder, then append the file (if file is null it appends index.html)
	 * @param uri
	 * @param file
	 * @return
	 */
	public static String appendFileIfUriIsPath(String uri, String file) {
	    if(uri.endsWith("/"))
	      uri = uri + (file != null ? file : "index.html");
	    return uri;
	}
	
    // get locale from uri

	public static String getLocaleFromUri(String uri, String valid, String method) {
		if("file".equals(method)) 
			return getLocaleFromUriByFile(uri, valid);
		else if("folder".equals(method)) 
			return getLocaleFromUriByFolder(uri, valid);
		else
			return null;
	}

	public static boolean isLocalizedUri(String uri, String locale, String method) {
		if("file".equals(method)) {
			return isLocalizedUriByFile(uri, locale);
		} else if("folder".equals(method)) {
			return isLocalizedUriByFolder(uri, locale);
		} else {
			return false;
		}
	}

	public static String getLocalizedUri(String uri, String locale, String method) {
		if("file".equals(method)) {
			return getLocalizedUriByFile(uri, locale);
		} else if("folder".equals(method)) {
			return getLocalizedUriByFolder(uri, locale);
		} else {
			return uri;
		}
	}
	
	public static String getLocaleFromUriByFolder(String uri, String valid) {
		// assumed to be on the form /en/index.html
		// If valid is not null it must be a comma-separated list of valid locales
		String locale;
		int slash = uri.indexOf('/',1); // skip the first one
		if(slash == -1)
			locale = uri.substring(1); // if that ever happens...
		else
			locale = uri.substring(1,slash);
		if(valid!=null) {
			if(valid.indexOf(locale) == -1)
				locale = null;
		}
		return locale;
	}
	
	public static boolean isLocalizedUriByFolder(String uri, String locale) {
		return ( locale!=null && locale.equals(getLocaleFromUriByFolder(uri, null)) );
	}
	
	public static String getLocaleFromUriByFile(String uri, String valid) {
		// assumed to be on the form .../index_en.html
		// If valid is not null it must be a comma-separated list of valid locales
		String locale;
		int slash = uri.lastIndexOf('/');
		int underscore = uri.lastIndexOf('_');
		if(underscore == -1 || underscore < slash) {
			locale = null;
		} else {
			int dot = uri.lastIndexOf('.');
			if(dot == -1)
				locale = uri.substring(underscore+1);
			else
				locale = uri.substring(underscore+1,dot);
			if(valid!=null) {
				if(valid.indexOf(locale) == -1)
					locale = null;
			} 
		}
		return locale;
	}
	
	public static boolean isLocalizedUriByFile(String uri, String locale) {
		return ( locale!=null && locale.equals(getLocaleFromUriByFile(uri, null)) );
	}
	
	public static String getLocalizedUriByFolder(String uri, String locale) {
		if(locale == null || locale.length()==0)
			return uri;
		return "/" + locale + uri;
	}
	
	public static String getLocalizedUriByFile(String uri, String locale) {
		if(locale == null || locale.length()==0)
			return uri;
		int slash = uri.lastIndexOf('/');
		int dot = uri.lastIndexOf('.');
		if(dot == -1 || dot < slash) {
			return uri + "_" + locale;
		} else {
			return uri.substring(0,dot) + "_" + locale + uri.substring(dot);
		}
	}

	public static String removeLocaleFromUriByFile(String uri, String valid) {
		// assumed to be on the form .../index_en.html
		// If valid is not null it must be a comma-separated list of valid locales
		String locale;
		int slash = uri.lastIndexOf('/');
		int underscore = uri.lastIndexOf('_');
		if(underscore == -1 || underscore < slash) {
			locale = null;
		} else {
			int dot = uri.lastIndexOf('.');
			if(dot == -1)
				locale = uri.substring(underscore+1);
			else
				locale = uri.substring(underscore+1,dot);
			if(valid!=null) {
				if(valid.indexOf(locale) == -1)
					locale = null;
			} 
		}
		if(locale==null) {
			return uri;
		} else {
			int ndx = uri.lastIndexOf("_"+locale);
			return uri.substring(0,ndx)+uri.substring(ndx+locale.length()+1);
		}
	}
}



