/*
 * Copyright (c) 2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision$
 * $LastChangedDate$
 * 
 */

package com.codedroids.oc.forms;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.ArrayList;
import javax.xml.parsers.ParserConfigurationException;
import java.sql.SQLException;
import java.text.ParseException;
import java.util.Collections;
import javax.mail.MessagingException;
import javax.mail.internet.InternetAddress;

import java.util.List;

import org.opencms.jsp.*;
import org.opencms.main.CmsException;
import org.opencms.mail.*;
import org.xml.sax.SAXException;

import com.codedroids.j.*;
import com.codedroids.records.*;
import com.codedroids.oc.records.*;
import com.codedroids.forms.*;
import com.codedroids.templating.*;
import com.codedroids.oc.templating.*;

import net.sf.json.JSONObject;

public class OCFormUtils {

	/**
	 * Takes format string like "foo:bar;and:one,two,${Title};..." and turn it into a map<br>
	 * No form for escape or quote is supported by this format<p>
	 * The example would return a Map like this (where "Some title" is an example for what could 
	 * be returned by <code>CmsJspActionElement.property("Title")</code> method):<br>
	 * <code>"foo"=["bar"]</code><br>
	 * <code>"and"=["one", "two", "Some title"]</code><br>
	 * @param cmsjsp
	 * @param formValuesStr 
	 * @return Map with the values as string arrays
	 */
	public static Map mapFromPropertyExpandedFormatString(CmsJspActionElement cmsjsp, String formValuesStr) {
		if(formValuesStr == null || formValuesStr.length()==0)
			return null;
		Map formValues = MapUtils.populateMapFromString(new HashMap(), formValuesStr, ";", ":", ",");
		Iterator ite = formValues.entrySet().iterator();
		while(ite.hasNext()) {
			Map.Entry entry = (Map.Entry)ite.next();
			String[] vs = (String[])entry.getValue();
			for(int vi=0; vi < vs.length; vi++) {
				String v = vs[vi];
				if(v.length() > 2 && v.startsWith("${") && v.endsWith("}")) {
					String pk = v.substring(1, v.length()-1);
					vs[vi] = cmsjsp.property(pk, null, v);
				}
			}
		}
		return formValues;
	}
	
	// Reserved persist config keys
	public static final String PCK_CLASS = "class";
	
	/**
	 * 
	 * @param form
	 * @param persistConfig
	 * @param cmsjsp
	 * @param pageContext
	 * @throws PersistFormToRecordException
	 */
	public static void persistFormToRecord(Form form, String persistConfig, CmsJspActionElement cmsjsp)
	throws PersistFormToRecordException
	{
		JSONObject config = JSONObject.fromString(persistConfig);
		String cname = null;
		if(config.has(PCK_CLASS))
			cname = config.optString(PCK_CLASS, null);
		else if(config.has(PersistViaJsp.PCK_JSP))
			cname = "com.codedroids.oc.forms.PersistViaJsp";
		if(cname==null)
			throw new PersistFormToRecordException("Configuration does not include class name - neither directly or indirectly via jsp-key");
		try {
		Class c = Class.forName(cname);
		I_PersistFormToRecord i = (I_PersistFormToRecord)c.newInstance();
		i.init(config, cmsjsp, cmsjsp.getJspContext());
		i.persist(form, config, cmsjsp, cmsjsp.getJspContext());
		i.finish(config, cmsjsp, cmsjsp.getJspContext());
		} catch (Exception e) {
			throw new PersistFormToRecordException("Failed to persist form: \"" + form + "\"", e);
		}
	}
	
	/**
	 * "Old-time" (aka overtaken by <code>persistFormToRecord()</code>) convenience for saving a record populated 
	 * with data from the <code>form</code>, where the record is specified by the <code>entityFile</code>. 
	 * Field names in entity and form must match for this to work.
	 * <p>For convenience a few specific fields are populated as well given the entity specifies the following fields:
	 * "creator" (String), "creationDate" (Timestamp) and "lastIp" (String). 
	 * @param form Populated form
	 * @param entityFile VFS path to an entity XML file
	 * @param cms Current CmsJspActionElement bean
	 * @param pageContext Current page contaxt. Used for setting the optional "lastIp" fields - may be null (in which case "lastIp" is never set)
	 * @throws ClassNotFoundException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 * @throws CmsException
	 * @throws InstantiationException
	 * @throws ParseException
	 * @throws SQLException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 */
	public static void saveFormToRecord(Form form, String entityFile, CmsJspActionElement cmsjsp) throws RecordException, ParseException
	{
		RecordFactory factory = OCRecordFactory.registerFactory(cmsjsp.getCmsObject(), entityFile);
        I_Record cd = factory.getNewInstance();
        form.pushToRecord(cd);
        // for convenience these are set if they exists
        if(cd.hasField("creator"))
          cd.takeValueForKey("creator", cmsjsp.user("name"));
        if(cd.hasField("creationDate"))
          cd.takeValueForKey("creationDate", new java.sql.Timestamp(System.currentTimeMillis()));
        if(cd.hasField("lastIp") && cmsjsp.getJspContext() != null)
          cd.takeValueForKey("lastIp", cmsjsp.getRequest().getRemoteHost());
        factory.saveRecord(cd);
        cd = null;
	}
	
	/**
	 * Runs the {@link #buildEmailsFromForm(List, Form, String, List, String, String, String, Map, CmsJspActionElement) buildEmailsFromForm} with the the form as "form" in root of context. 
	 * 
	 * @param form
	 * @param mails
	 * @param from 
	 * @param tos
	 * @param subject
	 * @param format
	 * @param template
	 * @param cmsjsp
	 * @return
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws InstantiationException
	 * @throws TemplatingException
	 * @throws MessagingException
	 */
	public static List buildEmailsFromForm(List mails, Form form, String from, List tos, String subject, String format, String template, CmsJspActionElement cmsjsp)
	throws IllegalAccessException, ClassNotFoundException, InstantiationException, TemplatingException, MessagingException
	{
		Map context = Collections.singletonMap("form", form.getValues());
		return buildEmailsFromForm(mails, form, from, tos, subject, format, template, context, cmsjsp);
	}
	
	/**
	 * Run the template engine per email address in the <code>tos</code> list using <code>context</code>
	 * as the root context for the template engine to produce the contents for the email.<br>
	 * The rest of the fields are used to populate fields of the email itself. 
	 * The <code>from</code>-parameter must be on <a href="http://www.ietf.org/rfc/rfc822.txt" target="_blank">RFC822</a> format,
	 * whereas the individual to-addresses currently should be just the address part (i.e. no personal name).  
	 * 
	 * <p>Note that the context is not used for the "legacy" templating methods (i.e. when the
	 * <code>format</code> is null or the <code>format</code> is not null, but the <code>template</code> is null.)<br>
	 * And vice versa, the <code>form</form> is not used if not using legacy settings, 
	 * in this case only the <code>context</code> is used.
	 * 
	 * <p><b>format</b>-string must be on JSON format, e.g.
	 * <br><code>{ "type":"xslt","mime":"text/html",... }</code> 
	 * <br>At least the <code>type</code> (template engine type) must be given. System setup determines the known types, 
	 * examples are "ftl", "xslt", "macro". <br>
	 * Other settings given by format depends on the particular template engine, typically these include a mime-setting, but if
	 * not this method will attempt to guess whether to send as HTML email or plain-text ditto.
	 * <br><i>If the format does not start with "{" then it is assumed to be the legacy format which 
	 * is 'type:xslt;...' - use of this format is deprecated</i>
	 * <p><b>template</b>-string gives the filename for the template which will be needed by the template
	 * engine specified in by the <b>format</b>
	 * <p>Two special cases:
	 * <br>1. if the <code>template</code> is null, then <b>format</b>-string is assumed to be the template itself, 
	 * where a simple macro substitution is performed. Example:
	 * <br><code>Dear ${firstName}\nThank you for your inquiry, we will contact as soon as possible</code>
	 * <br>where ${firstName} will be replaced by the value of the firstName field in the form 
	 * (if such exists, otherwise the macro is not substituted). Note that strings '\n' and '\t' are handled and included  
	 * the email as the corresponding control codes.
	 * <br>2. if both <code>template</code> and <code>format</code> are null, the content of the mail is a pretty printed
	 *  list of all fields (using the translated field label if present, otherwise it uses the field name):
	 * <br><code>First name: John</code>
	 * <br><code>Last name.: Doe</code>
	 * <br><code>Age.......: 35</code>
	 * <br><em>Note there is one exception, if the field name starts with an underscore '_' then it is not included in the email, 
	 * use this to hide certain field (or use one of the templating engines where you have full control over what fields to include).</p>
	 * 
	 * @param form
	 * @param mails
	 * @param from 
	 * @param tos
	 * @param subject
	 * @param format
	 * @param template
	 * @param context
	 * @param cmsjsp
	 * @return
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws InstantiationException
	 * @throws TemplatingException
	 * @throws MessagingException
	 */
	public static List buildEmailsFromForm(List mails, Form form, String from, List tos, String subject, String format, String template, Map context, CmsJspActionElement cmsjsp)
	throws IllegalAccessException, ClassNotFoundException, InstantiationException, TemplatingException, MessagingException
	{
		if(mails == null)
			mails = new ArrayList();
        String content = "";
        boolean htmlMail = false;

		if(format==null) {
			content = form.getFieldValuesAsLabeledText(null, "_");
			htmlMail = false;
		} else {
			if(template==null) {
				// old-style template then, 
				Map params = form.getFieldValues(null, ", ", " & ");
			    format = format.replaceAll("\\\\n", "\n");
			    format = format.replaceAll("\\\\t", "\t");
				content = StringUtils.replaceStrings(format, params, "${", "}");
				htmlMail = (content.indexOf("<html") >= 0); // a tad simplistic...
			} else {
				// flexible templating
				A_TemplateEngine te = OCTemplateEngineFactory.templateEngine(cmsjsp, template, format);
				StringWriter sw = new StringWriter();
				te.process(context, sw);
				content = sw.toString();
				String mime = te.getConfiguration().optString("mime", null);
				if(mime != null)
					htmlMail = mime.equals("text/html");
				else
					htmlMail = (content.indexOf("<html") >= 0);
			}
		}

		// Send them as individual emails
		InternetAddress fromAddr = new InternetAddress(from);
		Iterator it = tos.iterator();
		while (it.hasNext()) {
			if(htmlMail) {
				CmsHtmlMail ma = new CmsHtmlMail();
				if(fromAddr.getPersonal()==null)
					ma.setFrom(fromAddr.getAddress());
				else
					ma.setFrom(fromAddr.getAddress(), fromAddr.getPersonal());
				ma.addTo((String)it.next());
				ma.setSubject(subject);
				ma.setHtmlMsg(content);
				mails.add(ma);
			} else {
				CmsSimpleMail ma = new CmsSimpleMail();
				if(fromAddr.getPersonal()==null)
					ma.setFrom(fromAddr.getAddress());
				else
					ma.setFrom(fromAddr.getAddress(), fromAddr.getPersonal());
				ma.addTo((String)it.next());
				ma.setSubject(subject);
				ma.setMsg(content);
				mails.add(ma);
			}
		}
		return mails;
	}
	
	/**
	 * Run the template engine per email address in the <code>tos</code> list using <code>context</code>
	 * as the root context for the template engine to produce the contents for the email.<br>
	 * The rest of the fields are used to populate fields of the email itself.
	 * 
	 * <p><b>format</b>-string must be on JSON format, e.g. '{ "type":"xslt","mime":"text/html",... }'. 
	 * At least the <code>type</code> (template engine type) must be given. System setup determines the known types, 
	 * examples are "ftl", "xslt", "macro". <br>
	 * Other settings given by format depends on the particular template engine, typically these include a mime-setting, but if
	 * not this method will attempt to guess whether to send as HTML email or plain-text ditto.
	 * 
	 * <p><i>Note, similar to <code>buildEmailsFromForm()</code> but stripped of support for the legacy <code>format</code> and <code>template</code>-variations.</i></p>
	 * 
	 * @param mails
	 * @param from
	 * @param tos
	 * @param subject
	 * @param format
	 * @param template
	 * @param context
	 * @param cmsjsp
	 * @return
	 */
	public static List buildEmails( List mails, String from, List tos, String subject, String format, String template, Map context, CmsJspActionElement cmsjsp)
	throws IllegalAccessException, ClassNotFoundException, InstantiationException, TemplatingException, MessagingException
	{
		if(mails == null)
			mails = new ArrayList();
        String content = "";
        boolean htmlMail = false;

        A_TemplateEngine te = OCTemplateEngineFactory.templateEngine(cmsjsp, template, format);
        StringWriter sw = new StringWriter();
        te.process(context, sw);
        content = sw.toString();
        String mime = te.getConfiguration().optString("mime", null);
        if(mime != null)
        	htmlMail = mime.equals("text/html");
        else
        	htmlMail = (content.indexOf("<html") >= 0);

        InternetAddress fromAddr = new InternetAddress(from);
        Iterator it = tos.iterator();
		while (it.hasNext()) {
			if(htmlMail) {
				CmsHtmlMail ma = new CmsHtmlMail();
				if(fromAddr.getPersonal()==null)
					ma.setFrom(fromAddr.getAddress());
				else
					ma.setFrom(fromAddr.getAddress(), fromAddr.getPersonal());
				ma.addTo((String)it.next());
				ma.setSubject(subject);
				ma.setHtmlMsg(content);
				mails.add(ma);
			} else {
				CmsSimpleMail ma = new CmsSimpleMail();
				if(fromAddr.getPersonal()==null)
					ma.setFrom(fromAddr.getAddress());
				else
					ma.setFrom(fromAddr.getAddress(), fromAddr.getPersonal());
				ma.addTo((String)it.next());
				ma.setSubject(subject);
				ma.setMsg(content);
				mails.add(ma);
			}
		}
		return mails;
	}
	
	/**
	 * Sends already prepared apache commons emails. Allows for pausing every <code>batchSize</code> email 
	 * (mostly an issue with large amount of emails, to ease the push on the SMTP server, but in case of 
	 * many emails this should really run in its own thread, or maybe even use an external mailing list setup - well, later...)
	 * 
	 * @param mails
	 * @param batchSize
	 * @param sleepMillis
	 * @throws MessagingException
	 * @throws InterruptedException
	 */
	public static void sendEmails(List mails, int batchSize, int sleepMillis)
	throws MessagingException, InterruptedException
	{
		if(mails != null) {
			int cnt = mails.size();
			for(int m = 0; m<cnt; m++) {
				((org.apache.commons.mail.Email)mails.get(m)).send();
				if(m % batchSize == 0)
					Thread.sleep(sleepMillis);
			}
		}
	}
}
